import 'dart:io';

import 'package:args/args.dart';
import 'package:args/command_runner.dart';
import 'package:globe_cli/src/utils/auth.dart';
import 'package:mason_logger/mason_logger.dart';
import 'package:meta/meta.dart';

import 'utils/api.dart';

typedef StartProcess = Future<Process> Function(
  String executable,
  List<String> arguments, {
  bool runInShell,
});

typedef RunProcess = Future<ProcessResult> Function(
  String executable,
  List<String> arguments, {
  bool runInShell,
});

abstract class BaseGlobeCommand extends Command<int> {
  BaseGlobeCommand({
    GlobeAuth? auth,
    GlobeApi? api,
    Logger? logger,
    StartProcess? startProcess,
    RunProcess? runProcess,
  })  : auth = auth ?? GlobeAuth(),
        api = api ?? GlobeApi(),
        logger = logger ?? Logger(),
        startProcess = startProcess ?? Process.start,
        runProcess = runProcess ?? Process.run;

  final GlobeAuth auth;
  final GlobeApi api;
  final Logger logger;
  final RunProcess runProcess;
  final StartProcess startProcess;

  /// [ArgResults] used for testing purposes only.
  @visibleForTesting
  ArgResults? testArgResults;

  /// [ArgResults] for the current command.
  ArgResults get results => testArgResults ?? argResults!;

  /// [CommandRunner] used for testing purposes only.
  @visibleForTesting
  CommandRunner<int>? testCommandRunner;

  @override
  CommandRunner<int>? get runner => testCommandRunner ?? super.runner;

  /// Checks for a valid auth session.
  int? requireAuth() {
    final session = auth.currentSession;
    if (session == null) {
      logger
        ..err('Please login to deploy a project.')
        ..err('Run `globe login` to login.');

      exit(0);
    }

    // TODO(ehesp): Check for JWT expiry and refresh if needed?

    return null;
  }
}
