import 'dart:async';

import 'package:globe_cli/src/command.dart';
import 'package:globe_cli/src/utils/http_server.dart';
import 'package:globe_cli/src/utils/open_url.dart';
import 'package:mason_logger/mason_logger.dart';

/// {@template login_command}
///
/// `deploy login`
/// Login to the Dart Deploy service.
/// {@endtemplate}
class LoginCommand extends BaseGlobeCommand {
  /// {@macro login_command}
  LoginCommand({
    super.api,
    super.auth,
    super.logger,
    super.runProcess,
    super.startProcess,
    GlobeHttpServer? httpServer,
  }) : _httpServer = httpServer ?? GlobeHttpServer();

  final GlobeHttpServer _httpServer;

  @override
  String get name => 'login';

  @override
  String get description => 'Login to the Dart Deploy service.';

  @override
  FutureOr<int> run() async {
    final session = auth.currentSession;

    if (session != null) {
      logger.info('You are already logged in.');
      return ExitCode.success.code;
    }

    final port = await _httpServer.findUnusedPort();
    final sessionFuture = _httpServer.getSessionToken(port: port);

    logger.info('Please authenticate via the opened browser window.');

    await openUrl(
        '${api.endpoint}/login/cli?callback=http://localhost:$port/callback');

    final sessionToken = await sessionFuture;

    if (sessionToken == null) {
      logger.err('Failed to login.');
      return ExitCode.software.code;
    }

    try {
      auth.login(jwt: sessionToken);
      logger.success('Successfully logged in.');
      return ExitCode.success.code;
    } catch (error) {
      logger.err(error.toString());
      return ExitCode.software.code;
    }
  }
}
