import 'package:globe_cli/src/command.dart';
import 'package:mason_logger/mason_logger.dart';

/// {@template logout_command}
///
/// `dart_deploy logout`
/// Logout of the current user.
/// {@endtemplate}
class LogoutCommand extends BaseGlobeCommand {
  /// {@macro logout_command}
  LogoutCommand({
    super.api,
    super.auth,
    super.logger,
    super.runProcess,
    super.startProcess,
  });

  @override
  String get description => 'Logout of the current user';

  @override
  String get name => 'logout';

  @override
  Future<int> run() async {
    final session = auth.currentSession;
    if (session == null) {
      logger.info('You are already logged out.');
      return ExitCode.success.code;
    }

    final logoutProgress = logger.progress('Logging out.');
    auth.logout();
    logoutProgress.complete();

    return ExitCode.success.code;
  }
}
