import 'dart:io';

import 'package:archive/archive_io.dart';

/// A utility class for creating archives from a [Directory].
class Archiver {
  final Directory directory;

  Archiver({
    required this.directory,
  });

  Future<List<int>> archive() async {
    final archive = Archive();
    final files = directory.listSync(recursive: true);

    for (final file in files) {
      if (file is File) {
        archive.addFile(
          ArchiveFile(
            file.path.replaceFirst('${directory.path}/', ''),
            file.lengthSync(),
            file.readAsBytesSync(),
          ),
        );
      }
    }

    final encoded = ZipEncoder().encode(archive);

    if (encoded == null) {
      throw Exception('Failed to encode archive.');
    }

    return encoded;
  }
}
