import 'dart:async';
import 'dart:io';

/// A utility class for creating a simple HTTP server.
class GlobeHttpServer {
  /// Starts a local HTTP server which listens for an incoming request
  /// to the `/callback` endpoint with a `session` query parameter.
  ///
  /// Once the request is received, the server will respond with a 200
  /// and close the connection.
  Future<String?> getSessionToken({
    required int port,
  }) async {
    final host = 'localhost';
    final server = await HttpServer.bind(host, port);
    final completer = Completer<String?>();

    server.listen((req) async {
      if (req.method == 'OPTIONS') {
        req.response.statusCode = 200;
        req.response.headers.add('Access-Control-Allow-Origin', '*');
        req.response.headers.add('Access-Control-Allow-Methods', '*');
        req.response.headers.add('Access-Control-Allow-Headers', '*');
        await req.response.close();
      }

      if (req.method == 'GET' && req.uri.path == '/callback') {
        req.response.statusCode = 200;
        req.response.headers.add('Access-Control-Allow-Origin', '*');
        req.response.headers.add('Access-Control-Allow-Methods', '*');
        req.response.headers.add('Access-Control-Allow-Headers', '*');
        await req.response.close();
        await server.close();
        completer.complete(req.uri.queryParameters['session']);
      }
    });

    return completer.future;
  }

  /// Finds an unused port on the local machine.
  Future<int> findUnusedPort() async {
    int port;
    ServerSocket socket;
    try {
      socket = await ServerSocket.bind(InternetAddress.loopbackIPv4, 0);
    } on SocketException {
      throw Exception('Failed to bind to a port');
    }
    port = socket.port;
    await socket.close();
    return port;
  }
}
