import 'package:globe_cli/src/utils/api.dart';
import 'package:mason_logger/mason_logger.dart';

Future<Organization> selectOrganization({
  required GlobeApi api,
  required Logger logger,
}) async {
  logger.detail('Fetching user organizations');
  final List<Organization> organizations = await api.getOrganizations();
  logger.detail('Found ${organizations.length} organizations');

  /// If there's only one, automatically select it.
  if (organizations.length == 1) {
    final organization = organizations.first;
    logger.detail('Automatically selecting ${organization.name}.');
    return organization;
  }

  // Otherwise let the user select one.
  final selectedOrg = logger.chooseOne(
    'Select an organization to deploy to',
    choices: organizations.map((o) => o.id).toList(),
    display: (choice) => organizations.firstWhere((o) => o.id == choice).name,
  );

  return organizations.firstWhere((o) => o.name == selectedOrg);
}

Future<Project> selectProject(
  Organization organization, {
  required GlobeApi api,
  required Logger logger,
}) async {
  logger.detail('Fetching organization projects');
  final projects = await api.getProjects(org: organization.id);
  logger.detail('Found ${projects.length} projects');

  Future<Project> createProject() async {
    final name = logger.prompt('Please your new project name');
    return await api.createProject(
      orgId: organization.id,
      name: name,
    );
  }

  if (projects.isEmpty) {
    logger.detail('No projects found, prompting to create a new project');
    return createProject();
  }

  const createSymbol = '__CREATE_PROJECT';

  // Select a project or create a new one.
  final selectedProject = logger.chooseOne(
    'Select a project to deploy to',
    choices: [createSymbol, ...projects.map((p) => p.id).toList()],
    display: (choice) {
      if (choice == createSymbol) {
        return 'Create a new project';
      }

      return projects.firstWhere((p) => p.id == choice).name;
    },
  );

  // If the user selected to create a new project, prompt to create one.
  if (selectedProject == createSymbol) {
    return createProject();
  }

  return projects.firstWhere((p) => p.id == selectedProject);
}
